package com.idiot.operationbackend.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.idiot.operationbackend.entity.AccountFans;
import com.idiot.operationbackend.mappers.AccountFansMapper;
import com.idiot.operationbackend.service.facade.AccountFansService;
import com.idiot.operationbackend.support.Constants;
import com.idiot.operationbackend.vo.NumberStatData;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author wang xiao
 * @date Created in 19:24 2020/9/10
 */
@Service
public class AccountFansServiceImpl extends ServiceImpl<AccountFansMapper, AccountFans>
        implements AccountFansService {



    @Override
    public AccountFans queryByAccountIdAndOpenId(String accountId, String openId) {
        return getOne(Wrappers.<AccountFans>lambdaQuery().eq(AccountFans::getAccountId,accountId)
                .eq(AccountFans::getOpenId,openId),false);
    }


    @Override
    public int countFansByAccountId(String accountId) {
        return count(Wrappers.<AccountFans>lambdaQuery().eq(AccountFans::getAccountId,accountId)
                .eq(AccountFans::getState,1));
    }


    @Override
    public List<AccountFans> queryFansByDateAndDate(String accountId, String startDate, String endDate) {
        long  end = Constants.toLocalDateTime(endDate).plusDays(1)
                .toEpochSecond(Constants.DEFAULT_ZONE);
        long  start =Constants.toLocalDateTime(endDate)
                .toEpochSecond(Constants.DEFAULT_ZONE);
        return list(Wrappers.<AccountFans>lambdaQuery()
                .eq(AccountFans::getAccountId,accountId)
                .eq(AccountFans::getState,1)
                .ge(AccountFans::getCreateTime,start)
                .le(AccountFans::getCreateTime,end));
    }


    @Override
    public List<NumberStatData> statByFansProperty(String accountId, String startDate, String endDate) {
        List<NumberStatData> numberStatData = null;
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            long start = Constants.toEpochSecond(Constants.toLocalDateTime(startDate));
            long end = Constants.toEpochSecond(Constants.toLocalDateTime(endDate).plusDays(1));
            numberStatData = baseMapper.statPropertyByDate(accountId,start,end);
        }else {
            numberStatData = baseMapper.statPropertyByAll(accountId);
        }
        return numberStatData;
    }


    @Override
    public Page<AccountFans> pageFans(String accountId, Integer sex, String city, String province, String tagId, String label, String startTime, String endTime, int page, int pageSize) {
        Page<AccountFans> queryPage = new Page<>(page,pageSize);
        LambdaQueryWrapper<AccountFans> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(AccountFans::getAccountId,accountId);
        if (Objects.nonNull(sex)) {
            queryWrapper.eq(AccountFans::getSex,sex);
        }
        if (StringUtils.isNotBlank(city)) {
            queryWrapper.eq(AccountFans::getCity,city);
        }
        if (StringUtils.isNotBlank(province)) {
            queryWrapper.eq(AccountFans::getProvince,province);
        }
        if (StringUtils.isNotBlank(label)) {
            queryWrapper.eq(AccountFans::getNickName,label).or()
                    .eq(AccountFans::getRemark,label);
        }
        if (StringUtils.isNotBlank(startTime)) {
           long start =  Constants.toLocalDateTime(startTime).toEpochSecond(Constants.DEFAULT_ZONE);
           queryWrapper.ge(AccountFans::getSubscribeTime,start);
        }
        if (StringUtils.isNotBlank(endTime)) {
            long end =  Constants.toLocalDateTime(endTime).plusDays(1).toEpochSecond(Constants.DEFAULT_ZONE);
            queryWrapper.le(AccountFans::getSubscribeTime,end);
        }
        if (StringUtils.isNotBlank(tagId)) {
            queryWrapper.last(" AND locate('"+tagId+"' , tag_id_list) ORDER BY subscribe_time DESC");
        }else {
            queryWrapper.orderByDesc(AccountFans::getSubscribeTime);
        }
        return page(queryPage,queryWrapper);
    }

    @Override
    public boolean updateFansRemark(String accountId, String openId, String remark) {
        return update(Wrappers.<AccountFans>lambdaUpdate()
                .eq(AccountFans::getAccountId,accountId)
                .eq(AccountFans::getOpenId,openId)
                .set(AccountFans::getRemark,remark)
        );
    }

    @Override
    public boolean addItemForTagList(List<String> openIds, String accountId, int wxId) {
        if (CollectionUtils.isEmpty(openIds)){
            return false;
        }
        int size = openIds.size();
        List<AccountFans> fans = new ArrayList<>(size);
        for (String openId : openIds) {
            AccountFans temp = queryByAccountIdAndOpenId(accountId,openId);
            if (Objects.isNull(temp)){
                continue;
            }
            JSONArray jsonArray = JSONArray.parseArray(temp.getTagIdList());
            jsonArray.add(wxId);
            temp.setTagIdList(jsonArray.toJSONString());
            temp.setUpdateTime(LocalDateTime.now().format(Constants.DATE_TIME_FORMATTER));
            fans.add(temp);

        }
        return updateBatchById(fans);
    }


    @Override
    public boolean removeItemForTagList(List<String> openIds, String accountId, int wxId) {
        if (CollectionUtils.isEmpty(openIds)){
            return false;
        }
        int size = openIds.size();
        List<AccountFans> fans = new ArrayList<>(size);
        for (String openId : openIds) {
            AccountFans temp = queryByAccountIdAndOpenId(accountId,openId);
            if (Objects.isNull(temp)){
                continue;
            }
            JSONArray jsonArray = JSONArray.parseArray(temp.getTagIdList());
            jsonArray.remove(wxId);
            temp.setTagIdList(jsonArray.toJSONString());
            temp.setUpdateTime(LocalDateTime.now().format(Constants.DATE_TIME_FORMATTER));
            fans.add(temp);

        }
        return updateBatchById(fans);
    }

    @Override
    public List<AccountFans> queryAccountFans(List<String> openIds, String accountId) {
        if (CollectionUtils.isEmpty(openIds)){
            return new ArrayList<>();
        }
       return list(Wrappers.<AccountFans>lambdaQuery()
               .eq(AccountFans::getAccountId,accountId)
               .in(AccountFans::getOpenId,openIds));
    }
}
