package com.idiot.operationbackend.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.google.common.cache.*;
import com.idiot.operationbackend.config.PlatformProperties;
import com.idiot.operationbackend.entity.Account;
import com.idiot.operationbackend.entity.AccountFans;
import com.idiot.operationbackend.entity.AccountTag;
import com.idiot.operationbackend.handler.SyncUserTask;
import com.idiot.operationbackend.service.facade.AccountFansService;
import com.idiot.operationbackend.service.facade.AccountService;
import com.idiot.operationbackend.service.facade.AccountTagService;
import com.idiot.operationbackend.service.facade.WeChatService;
import com.idiot.operationbackend.support.Constants;
import com.idiot.operationbackend.support.CustomException;
import com.idiot.operationbackend.support.wechat.AesException;
import com.idiot.operationbackend.support.wechat.WXBizMsgCrypt;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;


/**
 * @author wang xiao
 * @date Created in 10:54 2020/9/11
 */
@Service
public class WeChatServiceImpl implements WeChatService, InitializingBean {


    private final Logger logger = LoggerFactory.getLogger(WeChatServiceImpl.class);

    private final Cache<String,String> cache = CacheBuilder.newBuilder()
            .concurrencyLevel(8)
            .expireAfterWrite(100, TimeUnit.MINUTES)
            .initialCapacity(10)
            .maximumSize(100)
            .recordStats()
            .removalListener(notification -> logger.warn("cache {} was removed, cause is {}" ,notification.getKey(), notification.getCause()))
            .build();

    private WXBizMsgCrypt pc;

    @Resource(name = "rpcRestTemplate")
    private RestTemplate restTemplate;


    @Autowired
    private PlatformProperties properties;

    @Autowired
    private AccountService accountService;

    @Autowired
    private AccountFansService fansService;

    @Autowired
    private AccountTagService tagService;


    @Override
    public String notice(Map<String, String> param) {
        if (Objects.isNull(param)) {
            return Constants.SUCCESS;
        }
        String infoType = param.get("InfoType");
        String componentAppId = param.get("AppId");
        String ticket = param.get("ComponentVerifyTicket");
        logger.info("接受微信验证票据数据,infoType:{},第三方平台appId:{},ticket:{}",infoType,componentAppId,ticket);
        cache.put(String.format(Constants.COMPONENT_VERIFY_TICKET,componentAppId),ticket);
        return Constants.SUCCESS;
    }


    @Override
    public String decryptMsg(String msgSignature, String timestamp, String nonce, String data)  {
        try {
            return pc.decryptMsg(msgSignature,timestamp,nonce,data);
        }catch (AesException e){
            logger.error("微信解密数据发生错误----->msgSignature:{},timestamp:{},nonce:{},data:{}",msgSignature,
                    timestamp,nonce,data);
            return null;
        }
    }

    @Override
    public String encryptMsg(String data) {
        String timestamp =  Long.toString(System.currentTimeMillis());
        String nonce = pc.getRandomStr();
        try {
            return  pc.encryptMsg(data,timestamp,nonce);
        }catch (AesException e){
            logger.error("微信加密数据发生错误----->timestamp:{},nonce:{},data:{}",timestamp,nonce,data);
            return null;
        }
    }

    @Override
    public void cacheAuthorizerAccessToken(String accountId, String authorizerAccessToken) {
        cache.put(String.format(Constants.AUTHORIZER_ACCESS_TOKEN,accountId),authorizerAccessToken);
    }

    @Override
    public boolean saveOrUpdateWechatAcc(Account account) {
        return accountService.saveOrUpdateAccount(account);
    }

    @Override
    public String getComponentAccessToken() {
        logger.info("获取第三方平台令牌 component_access_token start");
        String componentAppId = properties.getAppId();
        String componentAccessToken = cache.getIfPresent(String.format(Constants.COMPONENT_ACCESS_TOKEN,componentAppId));
        if ( !StringUtils.isEmpty(componentAccessToken)) {
            return componentAccessToken;
        }
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/component/api_component_token";
        HttpHeaders headers = new HttpHeaders();
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(3);
        headers.setContentType(MediaType.APPLICATION_JSON);
        String ticket = cache.getIfPresent(String.format(Constants.COMPONENT_VERIFY_TICKET,componentAppId));
        params.add("component_appid",componentAppId);
        params.add("component_appsecret",properties.getAppSecret());
        params.add("component_verify_ticket",ticket);
        HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(params, headers);
        ResponseEntity<String> response = restTemplate.exchange(requestUrl, HttpMethod.POST, requestEntity, String.class);
        String  jsonStr = response.getBody();
        logger.info("请求微信获取令牌component_access_token 第三方平台appId:{},ticket:{},微信返回结果:{}",componentAppId,ticket,jsonStr);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        componentAccessToken = jsonObject.getString("component_access_token");
        cache.put(String.format(Constants.COMPONENT_ACCESS_TOKEN,componentAppId),componentAccessToken);
        return componentAccessToken;
    }

    @Override
    public String getAuthorizerAccessToken(String accountId) {

        logger.info("获取授权方令牌 authorizer_access_token start");
        String authorizerAccessToken = cache.getIfPresent(String.format(Constants.AUTHORIZER_ACCESS_TOKEN,accountId));
        if ( !StringUtils.isEmpty(authorizerAccessToken)) {
            return authorizerAccessToken;
        }
        Account account = accountService.getById(accountId);
        if (Objects.isNull(account)) {
            logger.error("获取授权方令牌 authorizer_access_token 发生异常===> 公众号id:{}为空",accountId);
            return null;
        }
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?component_access_token=COMPONENT_TOKEN";
        String componentAccessToken = getComponentAccessToken();
        String componentAppId = properties.getAppId();
        requestUrl = requestUrl.replace("COMPONENT_TOKEN",componentAccessToken);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(4);
        params.add("component_appid",componentAppId);
        params.add("component_access_token",componentAccessToken);
        params.add("authorizer_appid",account.getAuthorizerAppId());
        params.add("authorizer_refresh_token",account.getAuthorizerRefreshToken());
        HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(params, headers);
        ResponseEntity<String> response = restTemplate.exchange(requestUrl, HttpMethod.POST, requestEntity, String.class);
        String  jsonStr = response.getBody();
        logger.info("获取授权方令牌 authorizer_access_token 第三方平台appId:{},公众号id:{},授权方appId:{},微信返回结果:{}",
                componentAppId,accountId,account.getAuthorizerAppId(),jsonStr);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        authorizerAccessToken = jsonObject.getString("authorizer_access_token");
        String authorizerRefreshToken  = jsonObject.getString("authorizer_refresh_token");
        account.setAuthorizerAccessToken(authorizerAccessToken);
        account.setAuthorizerRefreshToken(authorizerRefreshToken);
        cache.put(String.format(Constants.AUTHORIZER_ACCESS_TOKEN,accountId),authorizerAccessToken);
        accountService.saveOrUpdate(account);
        return authorizerAccessToken;
    }

    @Override
    public String getPreAuthCode() {
        String componentAppId = properties.getAppId();
        logger.info("请求微信获取预授权码 第三方平台appId:{} start",componentAppId);
        String requestUrl =  "https://api.weixin.qq.com/cgi-bin/component/api_create_preauthcode?component_access_token=COMPONENT_ACCESS_TOKEN";
        String componentAccessToken = getComponentAccessToken();
        requestUrl = requestUrl.replace("COMPONENT_ACCESS_TOKEN",componentAccessToken);
        HttpHeaders headers = new HttpHeaders();
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(1);
        params.add("component_appid",componentAppId);
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(params, headers);
        ResponseEntity<String> response = restTemplate.exchange(requestUrl, HttpMethod.POST, requestEntity, String.class);
        String  jsonStr = response.getBody();
        logger.info("请求微信获取预授权码 第三方平台appId:{},微信返回结果:{}",componentAppId,jsonStr);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        return jsonObject.getString("pre_auth_code");
    }

    @Override
    public String getPreAuthUrl() {
        String preAuthUrl = "https://mp.weixin.qq.com/cgi-bin/componentloginpage?component_appid=APPID&pre_auth_code=PREAUTHCODE&redirect_uri=URI&auth_type=1";
        String componentAppId = properties.getAppId();
        String preAuthCode = getPreAuthCode();
        preAuthUrl = preAuthUrl.replace("APPID",componentAppId).replace("PREAUTHCODE",preAuthCode);
        logger.info("生成预授权地址, preAuthUrl is {}",preAuthUrl);
        return preAuthUrl;
    }


    @Override
    public void afterPropertiesSet() throws Exception {
        logger.info("WeChatService类--------------->正在初始化");
        pc = new WXBizMsgCrypt(properties.getToken(), properties.getSecret(), properties.getAppId());
        logger.info("cache size  is {}",cache.size());
        logger.info("WeChatService类--------------->初始化完成");
    }


    @Override
    public String getAuthorizationInfo(String authCode) {
        String componentAppId = properties.getAppId();
        logger.info("请求微信获取授权信息 第三方平台appId:{} start",componentAppId);
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/component/api_query_auth?component_access_token=COMPONENT_TOKEN";
        String componentToken = getComponentAccessToken();
        requestUrl = requestUrl.replace("COMPONENT_TOKEN",componentToken);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        MultiValueMap<String,String> multiValueMap = new LinkedMultiValueMap<>(2);
        multiValueMap.add("component_appid",componentAppId);
        multiValueMap.add("authorization_code",authCode);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (multiValueMap,headers);
        ResponseEntity<String> response = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = response.getBody();
        logger.info("请求微信获取授权信息 第三方平台appId:{},微信返回结果:{}",componentAppId,jsonStr);
        return jsonStr;
    }

    @Override
    public String getAuthorizerInfo(String authorizerAppId) {
        String componentAppId = properties.getAppId();
        logger.info("请求微信获取授权方信息 第三方平台appId:{},授权方appId:{},start",componentAppId,authorizerAppId);
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/component/api_get_authorizer_info?component_access_token=COMPONENT_TOKEN";
        String componentToken = getComponentAccessToken();
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        requestUrl = requestUrl.replace("COMPONENT_TOKEN",componentToken);
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(4);
        params.add("component_access_token",componentToken);
        params.add("component_appid",componentAppId);
        params.add("authorizer_appid",authorizerAppId);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (params,headers);
        ResponseEntity<String> response = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = response.getBody();
        logger.info("请求微信获取授权方信息 第三方平台appId:{},授权方appId:{},微信返回结果:{}",componentAppId,authorizerAppId,jsonStr);
        return jsonStr;
    }


    @Override
    public int syncAccountUser(String accountId) {

        String lockKey = String.format(Constants.LOCK_SYNC_USER,accountId);
        String lockValue = cache.getIfPresent(lockKey);
        if (!StringUtils.isEmpty(lockValue)) {
            throw new CustomException(500,"当前公众号正在后台同步粉丝数据,请您稍等一会!");
        }
        cache.put(lockKey,lockKey);
        logger.info("公众号：{}同步粉丝数据---- start,时间:{}",accountId, LocalDateTime.now().toString());
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/user/get?access_token=%s&next_openid=%s";
        String nextOpenId = "";
        String accessToken = getAuthorizerAccessToken(accountId);
        int count = 0;
        int total = 0;
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (headers);
        String jsonStr = null;
        JSONObject respJsonBody = null;
        do {
            if (StringUtils.isEmpty(nextOpenId)) {
                nextOpenId = "";
            }
            requestUrl = String.format(requestUrl,accessToken,nextOpenId);
            ResponseEntity<String> responseEntity = restTemplate.exchange(requestUrl,HttpMethod.GET,entity,String.class);
            jsonStr = responseEntity.getBody();
            respJsonBody = JSONObject.parseObject(jsonStr);
            total = respJsonBody.getInteger("total");
            count += respJsonBody.getInteger("count");
            nextOpenId = respJsonBody.getString("next_openid");
            JSONArray openIdArray = respJsonBody.getJSONObject("data").getJSONArray("openid");
            List<String> openIds = JSONArray.parseArray(openIdArray.toJSONString(),String.class);
            new SyncUserTask(accountId,openIds,this);
        }while (count < total);
        cache.invalidate(lockKey);
        logger.info("公众号：{}同步粉丝数据----end,时间:{}",accountId, LocalDateTime.now().toString());
        return count;
    }


    @Override
    @Async("asyncExecutor")
    public void syncUserTask(String accountId, List<String> openIds) {

        if (CollectionUtils.isEmpty(openIds)) {
            return;
        }
        int size = openIds.size();
        logger.info("公众号：{}同步粉丝请求粉丝信息,openId 大小:{}---- start,时间:{}",accountId,size,LocalDateTime.now().toString());
        List<AccountFans> accountFans =  new ArrayList<>(size);

        for (String openId : openIds) {
            AccountFans fans = fansService.queryByAccountIdAndOpenId(accountId,openId);
            if (fans == null) {
                fans = new AccountFans();
            }
            JSONObject fansObject = JSON.parseObject(getFansInfo(accountId,openId));
            fans.setAccountId(accountId);
            fans.setOpenId(openId);
            fans.setNickName(fansObject.getString("nickname"));
            fans.setHeadImgUrl(fansObject.getString("headimgurl"));
            fans.setSex(fansObject.getInteger("sex"));
            fans.setSubscribe(fansObject.getInteger("subscribe"));
            fans.setCity(fansObject.getString("city"));
            fans.setProvince(fansObject.getString("province"));
            fans.setSubscribeTime(fansObject.getLong("subscribe_time"));
            fans.setSubscribeScene(fansObject.getString("subscribe_scene"));
            fans.setUnionId(fansObject.getString("unionid"));
            fans.setRemark(fansObject.getString("remark"));
            fans.setGroupId(fansObject.getInteger("groupid"));
            fans.setTagIdList(JSONObject.toJSONString(fansObject.getJSONObject("tagid_list")));
            accountFans.add(fans);
        }
        logger.info("公众号：{}同步粉丝请求粉丝信息,openId 大小:{}---- end,时间:{}",accountId,size,LocalDateTime.now().toString());
        //  保存用户
        fansService.saveOrUpdateBatch(accountFans,1000);
    }


    @Override
    @Async("asyncExecutor")
    public void syncTag(String accountId) {

        String lockKey = String.format(Constants.LOCK_SYNC_TAG,accountId);
        String cacheValue = cache.getIfPresent(lockKey);
        if (!StringUtils.isEmpty(cacheValue)) {
            throw new CustomException(500,"");
        }
        cache.put(lockKey,lockKey);
        logger.info("公众号：{}同步标签 ---- start,时间:{}",accountId,LocalDateTime.now().toString());
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/tags/get?access_token=%s";
        String accessToken =  getAuthorizerAccessToken(accountId);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        requestUrl = String.format(requestUrl,accessToken);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (headers);
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.GET,entity,String.class);
        JSONArray jsonArray = JSONObject.parseObject(respStr.getBody()).getJSONArray("tags");
        int size = jsonArray.size();
        List<AccountTag> accountTags = new ArrayList<>(size);
        JSONObject temp = null;
        AccountTag tempTag = null;
        String createTime  = Constants.DATE_TIME_FORMATTER.format(LocalDateTime.now());
        for (int i = 0; i < size; i++) {
            temp = jsonArray.getJSONObject(i);
            tempTag = new AccountTag();
            tempTag.setAccountId(accountId);
            tempTag.setFansCount(temp.getInteger("count"));
            tempTag.setName(temp.getString("name"));
            tempTag.setWxId(temp.getInteger("id"));
            tempTag.setCreateTime(createTime);
            accountTags.add(tempTag);
        }
        logger.info("公众号：{}同步标签 ---- end,时间:{}",accountId,LocalDateTime.now().toString());
        // 新增 多余的 删掉缺少的 俗称覆盖
        boolean ifOverlay = tagService.overlayAccountTag(accountId,accountTags);
        cache.invalidate(lockKey);

    }

    @Override
    public String getFansInfo(String accountId, String openId) {
        logger.info("查询粉丝信息，openId：{}----start,时间:{}",accountId, LocalDateTime.now().toString());
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/user/info?access_token=%s&openid=OPENID&lang=zh_CN";
        String accessToken =  getAuthorizerAccessToken(accountId);
        requestUrl = String.format(requestUrl,accessToken);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (headers);
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.GET,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("查询粉丝信息，openId：{}----end,时间:{},微信返回:{}",accountId, LocalDateTime.now().toString(),jsonStr);
        return jsonStr;
    }


    @Override
    public boolean confirmAccount(String accountId, String userId) {
        return accountService.updateUserIdByAccount(accountId,userId);
    }

    @Override
    public String getFansSunmmary(String accountId,String startDate,String endDate) {
        String requestUrl = "https://api.weixin.qq.com/datacube/getusersummary?access_token=%s";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl = String.format(requestUrl,accessToken);
        logger.info("获取微信用户增减数据，accountId：{}----start,时间:{}",accountId, LocalDateTime.now().toString());
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(2);
        params.add("begin_date",startDate);
        params.add("end_date",endDate);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (params,headers);
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("获取微信用户增减数据，accountId：{}----end,时间:{},微信返回{}",accountId,
                LocalDateTime.now().toString(),jsonStr);
        return jsonStr;
    }


    @Override
    public String getFansCumulate(String accountId, String startDate, String endDate) {
        String requestUrl = "https://api.weixin.qq.com/datacube/getusercumulate?access_token=%s";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl = String.format(requestUrl,accessToken);
        logger.info("获取微信用户汇总数据，accountId：{}----start,时间:{}",accountId, LocalDateTime.now().toString());
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(2);
        params.add("begin_date",startDate);
        params.add("end_date",endDate);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (params,headers);
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("获取微信用户汇总数据，accountId：{}----end,时间:{},微信返回{}",accountId,
                LocalDateTime.now().toString(),jsonStr);
        return null;
    }

    @Override
    public String getArticleSummary(String accountId, String startDate, String endDate) {
        String requestUrl = "https://api.weixin.qq.com/datacube/getarticlesummary?access_token=%s";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl = String.format(requestUrl,accessToken);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        logger.info("获取微信图文阅读数据，accountId：{}----start,时间:{}",accountId, LocalDateTime.now().toString());
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(2);
        params.add("begin_date",startDate);
        params.add("end_date",endDate);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (params,headers);
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("获取微信图文阅读数据，accountId：{}----end,时间:{},微信返回{}",accountId,
                LocalDateTime.now().toString(),jsonStr);
        return jsonStr;
    }

    @Override
    public boolean updateRemark(String accountId, String openId, String remark) {
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/user/info/updateremark?access_token=%s";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl = String.format(requestUrl,accessToken);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        logger.info("微信设置备注，accountId：{}----start,时间:{}",accountId, LocalDateTime.now().toString());
        MultiValueMap<String,String> params = new LinkedMultiValueMap<>(2);
        params.add("openid",openId);
        params.add("remark",remark);
        HttpEntity<MultiValueMap<String, String>>  entity = new HttpEntity<> (params,headers);
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("微信设置备注，accountId：{}----end,时间:{},微信返回{}",accountId,LocalDateTime.now().toString(),jsonStr);
        if (0 ==JSONObject.parseObject(jsonStr).getIntValue("errcode")) {
            return fansService.updateFansRemark(accountId,openId,remark);
        }
        return false;
    }

    @Override
    public boolean createTag(String accountId, String tagLabel) {

        String requestUrl = "https://api.weixin.qq.com/cgi-bin/tags/create?access_token=%s";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl =  String.format(requestUrl,accessToken);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        MultiValueMap<String,Object> params = new LinkedMultiValueMap<>(2);
        HashMap<String,String> map = new HashMap<>(2);
        map.put("name",tagLabel);
        params.add("tag",map);
        HttpEntity<MultiValueMap<String, Object>>  entity = new HttpEntity<> (params,headers);
        logger.info("微信创建标签，accountId：{}----start,时间:{}",accountId, LocalDateTime.now().toString());
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("微信创建标签，accountId：{}----end,时间:{},微信返回{}",accountId,LocalDateTime.now().toString(),jsonStr);
        JSONObject temp = JSONObject.parseObject(jsonStr).getJSONObject("tag");
        if (null != temp) {
            Integer wxId = temp.getInteger("id");
            return tagService.addTag(tagLabel,accountId,wxId);
        }
        return false;
    }


    @Override
    public boolean addUserTag(String accountId, List<String> openIds, int wxId) {
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/tags/members/batchtagging?access_token=%s";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl =  String.format(requestUrl,accessToken);

        MultiValueMap<String,Object> params = new LinkedMultiValueMap<>(2);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        params.add("openid_list",openIds);
        params.add("tagid",wxId);
        int size = openIds.size();
        HttpEntity<MultiValueMap<String, Object>>  entity = new HttpEntity<> (params,headers);
        logger.info("微信用户打标签，accountId：{},opedId size:{}----start,时间:{}",accountId,size, LocalDateTime.now().toString());
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("微信用户打标签，accountId：{},opedId size:{}----end,时间:{},微信返回：{}",accountId,size, LocalDateTime.now().toString(),jsonStr);
        if (0 ==JSONObject.parseObject(jsonStr).getIntValue("errcode")) {
            return tagService.addTagSize(accountId,wxId,size);
        }
        return false;
    }


    @Override
    public boolean removeUserTag(String accountId, List<String> openIds, int wxId) {
        String requestUrl = "https://api.weixin.qq.com/cgi-bin/tags/members/batchuntagging?access_token=%s";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl =  String.format(requestUrl,accessToken);
        MultiValueMap<String,Object> params = new LinkedMultiValueMap<>(2);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        params.add("openid_list",openIds);
        params.add("tagid",wxId);
        int size = openIds.size();
        HttpEntity<MultiValueMap<String, Object>>  entity = new HttpEntity<> (params,headers);
        logger.info("微信用户去除标签，accountId：{},opedId size:{}----start,时间:{}",accountId,size, LocalDateTime.now().toString());
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("微信用去除标签，accountId：{},opedId size:{}----end,时间:{},微信返回：{}",accountId,size, LocalDateTime.now().toString(),jsonStr);
        if (0 ==JSONObject.parseObject(jsonStr).getIntValue("errcode")) {
            return tagService.subTagSize(accountId,wxId,size);
        }
        return false;
    }


    @Override
    public boolean upTag(String accountId, int wxId, String name) {

        String requestUrl = "https://api.weixin.qq.com/cgi-bin/tags/update?access_token=ACCESS_TOKEN";
        String accessToken = getAuthorizerAccessToken(accountId);
        requestUrl =  String.format(requestUrl,accessToken);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        MultiValueMap<String,Object> params = new LinkedMultiValueMap<>(2);
        HashMap<String,Object> map = new HashMap<>(2);
        map.put("name",name);
        map.put("id",wxId);
        params.add("tag",map);
        HttpEntity<MultiValueMap<String, Object>>  entity = new HttpEntity<> (params,headers);
        logger.info("微信创建标签，accountId：{}----start,时间:{}",accountId, LocalDateTime.now().toString());
        ResponseEntity<String> respStr = restTemplate.exchange(requestUrl,HttpMethod.POST,entity,String.class);
        String jsonStr = respStr.getBody();
        logger.info("微信创建标签，accountId：{}----end,时间:{},微信返回{}",accountId,LocalDateTime.now().toString(),jsonStr);
        int errorCode = JSONObject.parseObject(jsonStr).getIntValue("errcode");
        return 0 == errorCode;
    }
}
